@@ -4,10 +4,17 @@ use Imager;
 use vars qw($VERSION @ISA);
 
 BEGIN {
-  $VERSION = "0.86";
+  $VERSION = "0.78";
 
-  require XSLoader;
-  XSLoader::load('Imager::File::TIFF', $VERSION);
+  eval {
+    require XSLoader;
+    XSLoader::load('Imager::File::TIFF', $VERSION);
+    1;
+  } or do {
+    require DynaLoader;
+    push @ISA, 'DynaLoader';
+    bootstrap Imager::File::TIFF $VERSION;
+  };
 }
 
 Imager->register_reader
@@ -124,7 +131,7 @@ Imager's TIFF support is documented in L<Imager::Files>.
 
 =head1 AUTHOR
 
-Tony Cook <tonyc@cpan.org>
+Tony Cook <tony@imager.perl.org>
 
 =head1 SEE ALSO
 
