@@ -5,23 +5,24 @@ use vars qw($VERSION @ISA);
 @ISA = qw(Imager::Font);
 
 BEGIN {
-  $VERSION = "0.87";
+  $VERSION = "0.78";
 
-  require XSLoader;
-  XSLoader::load('Imager::Font::W32', $VERSION);
+  eval {
+    require XSLoader;
+    XSLoader::load('Imager::Font::W32', $VERSION);
+    1;
+  } or do {
+    require DynaLoader;
+    push @ISA, 'DynaLoader';
+    bootstrap Imager::Font::W32 $VERSION;
+  };
 }
 
 # called by Imager::Font::new()
 # since Win32's HFONTs include the size information this
 # is just a stub
 sub new {
-  my $class = shift;
-  my %opts =
-      (
-       color => Imager::Color->new(255, 0, 0),
-       size => 15,
-       @_,
-      );
+  my ($class, %opts) = @_;
 
   return bless \%opts, $class;
 }
@@ -30,12 +31,6 @@ sub _bounding_box {
   my ($self, %opts) = @_;
   
   my @bbox = i_wf_bbox($self->{face}, $opts{size}, $opts{string}, $opts{utf8});
-  unless (@bbox) {
-    Imager->_set_error(Imager->_error_as_msg);
-    return;
-  }
-
-  return @bbox;
 }
 
 sub _draw {
@@ -43,12 +38,12 @@ sub _draw {
 
   my %input = @_;
   if (exists $input{channel}) {
-    return i_wf_cp($self->{face}, $input{image}{IMG}, $input{x}, $input{'y'},
+    i_wf_cp($self->{face}, $input{image}{IMG}, $input{x}, $input{'y'},
 	    $input{channel}, $input{size},
 	    $input{string}, $input{align}, $input{aa}, $input{utf8});
   }
   else {
-    return i_wf_text($self->{face}, $input{image}{IMG}, $input{x}, 
+    i_wf_text($self->{face}, $input{image}{IMG}, $input{x}, 
 	      $input{'y'}, $input{color}, $input{size}, 
 	      $input{string}, $input{align}, $input{aa}, $input{utf8});
   }
@@ -59,10 +54,6 @@ sub utf8 {
   return 1;
 }
 
-sub can_glyph_names {
-  return;
-}
-
 1;
 
 __END__
@@ -94,7 +85,7 @@ renamed it.
 
 =head1 AUTHOR
 
-Tony Cook <tonyc@cpan.org>
+Tony Cook <tony@imager.perl.org>
 
 =head1 SEE ALSO
 
